//notation: js file can only use this kind of comments
//since comments will cause error when use in webview.loadurl,
//comments will be remove by java use regexp
(function () {
	if (window.SoftbankJsSdk) {
		return;
	}
	var messagingIframe;
	var sendMessageQueue = [];
	var receiveMessageQueue = [];
	var messageHandlers = {};

	var CUSTOM_PROTOCOL_SCHEME = 'yy';
	var QUEUE_HAS_MESSAGE = '__QUEUE_MESSAGE__/';

	var responseCallbacks = {};
	var uniqueId = 1;

	function _createQueueReadyIframe(doc) {
		messagingIframe = doc.createElement('iframe');
		messagingIframe.style.display = 'none';
		doc.documentElement.appendChild(messagingIframe);
	}

	//set default messageHandler
	function init(messageHandler) {
		SoftbankJsSdk._messageHandler = messageHandler;
		var receivedMessages = receiveMessageQueue;
	    if (receivedMessages && receivedMessages != undefined) {
			for (var i = 0; i < receivedMessages.length; i++) {
				_dispatchMessageFromNative(receivedMessages[i]);
			}
		}
		receiveMessageQueue = null;
	}

	function send(data, responseCallback) {
		_doSend({
			data: data
		}, responseCallback);
	}

	function registerHandler(handlerName, handler) {
		messageHandlers[handlerName] = handler;
	}

	function callHandler(handlerName, data, responseCallback) {
		_doSend({
			handlerName: handlerName,
			data: data
		}, responseCallback);
	}

	//sendMessage add message, 触发native处理 sendMessage
	function _doSend(message, responseCallback) {
		if (responseCallback) {
			var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
			responseCallbacks[callbackId] = responseCallback;
			message.callbackId = callbackId;
		}

		sendMessageQueue.push(message);
		//通知native调用 _fetchQueue 方法（详细见BridgeWebViewClient的shouldOverrideUrlLoading方法）
		messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;
	}

	// 提供给native调用,该函数作用:获取sendMessageQueue返回给native,由于android不能直接获取返回的内容,所以使用url shouldOverrideUrlLoading 的方式返回内容
	function _fetchQueue() {
		var messageQueueString = JSON.stringify(sendMessageQueue);
		sendMessageQueue = [];
		//android can't read directly the return data, so we can reload iframe src to communicate with java
		messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://return/_fetchQueue/' + encodeURIComponent(messageQueueString);
	}

	//提供给native使用,
	function _dispatchMessageFromNative(messageJSON) {
		setTimeout(function () {
			var message = JSON.parse(messageJSON);
			var responseCallback;
			//java call finished, now need to call js callback function
			if (message.responseId) {
				responseCallback = responseCallbacks[message.responseId];
				if (!responseCallback) {
					return;
				}
				//responseCallback(message.responseData);
				//直接返回Json对象
				responseCallback(JSON.parse(message.responseData));
				delete responseCallbacks[message.responseId];
			} else {
				//直接发送
				if (message.callbackId) {
					var callbackResponseId = message.callbackId;
					responseCallback = function (responseData) {
						_doSend({
							responseId: callbackResponseId,
							responseData: responseData
						});
					};
				}

				var handler = SoftbankJsSdk._messageHandler;
				if (message.handlerName) {
					handler = messageHandlers[message.handlerName];
				}
				//查找指定handler
				try {
					//                    handler(message.data, responseCallback);
					//直接返回Json对象
					handler(JSON.parse(message.data), responseCallback);
				} catch (exception) {
					if (typeof console != 'undefined') {
						console.log("SoftbankJsSdk: WARNING: javascript handler threw.", message, exception);
					}
				}
			}
		});
	}

	//提供给native调用,receiveMessageQueue 在会在页面加载完后赋值为null,所以
	function _handleMessageFromNative(messageJSON) {
//		console.log(messageJSON);
        messageJSON = decodeURIComponent(window.atob(messageJSON));
		if (receiveMessageQueue && receiveMessageQueue.length > 0) {
			receiveMessageQueue.push(messageJSON);
		} else {
			_dispatchMessageFromNative(messageJSON);
		}
	}

	function getSdkVersion(responseCallback) {
		_doSend({
			handlerName: 'getSdkVersion'
		}, responseCallback);
	}

	function scanQRCode(responseCallback) {
		var data = {};
		_doSend({
			handlerName: 'scanQRCode',
			data: data
		}, responseCallback);
	}
	//身份证扫描
	function scanIdCard(responseCallback) {
	    var data = {};
        _doSend({
            handlerName: 'scanIdCard',
            data: data
        }, responseCallback);
	}

    //银行卡扫描
    function scanBackCard(responseCallback) {
        var data = {};
        _doSend({
            handlerName: 'scanBankCard',
            data: data
        }, responseCallback);
    }

     //跳转指定Activity页面。
    function showSpecifyActivity(responseCallback) {
        var data = {};
        _doSend({
            handlerName: 'showSpecifyActivity',
            data: data
        }, responseCallback);
    }

    //身份验证
    function identifyVerify(responseCallback) {
        var data = {};
        _doSend({
            handlerName: 'identifyVerify',
            data: data
        }, responseCallback);
    }

	function showKeyboard(data) {
		_doSend({
			handlerName: 'showKeyboard',
			data: data
		}, null);
	}

	function closeKeyboard() {
		_doSend({
			handlerName: 'closeKeyboard'
		}, null);
	}

	function closeWindow() {
		_doSend({
			handlerName: 'closeWindow'
		}, null);
	}

	function initStyle(data) {
		_doSend({
			handlerName: 'initStyle',
			data: data
		}, null);
	}

	function goBackOrForward(data) {
		_doSend({
			handlerName: 'goBackOrForward',
			data: data
		}, null);
	}

	function setSoftKeyboardInputListener(listener) {
		registerHandler("onTextChanged", function (data, responseCallback) {
			// inputLength = data.inputLength;
			// password = data.password;
			listener(data);
		});
	}

	function getBankBaseInfo(responseCallback) {
		_doSend({
			handlerName: 'getBankBaseInfo'
		}, responseCallback);
	}

	function requestToken(responseCallback) {
		_doSend({
			handlerName: 'requestToken',
		}, responseCallback);
	}

	function logoutPer(responseCallback) {
        _doSend({
            handlerName: 'logoutPer',
        }, responseCallback);
    }

    function blackBoxInfoback(responseCallback){
        _doSend({
            handlerName: 'blackBoxInfoback',
        }, responseCallback);
    }

	function requestTokenOrRedirect(data, responseCallback) {
		_doSend({
			handlerName: 'requestTokenOrRedirect',
			data: data
		}, responseCallback);
	}

	function checkLogonState(responseCallback) {
		var data = {};
		_doSend({
			handlerName: 'checkLogonState',
			data: data
		}, responseCallback);
	}

	function requestLogonAndRedirect(data) {
		_doSend({
			handlerName: 'requestLogonAndRedirect',
			data: data
		}, null);
	}

	function post(data, responseCallback) {
		_doSend({
			handlerName: 'post',
			data: data
		}, responseCallback);
	}

	function share(data) {
		_doSend({
			handlerName: 'share',
			data: data
		}, null);
	}

	function doLivenessDetect(data, responseCallback) {
		_doSend({
			handlerName: 'doLivenessDetect',
			data: data
		}, responseCallback);
	}

	function push(data) {
		_doSend({
			handlerName: 'push',
			data: data
		}, null);
	}

	function gainPayPassword(data, responseCallback) {
		_doSend({
			handlerName: 'gainPayPassword',
			data: data
		}, responseCallback);
	}

	function checkSoftCert(responseCallback) {
    		_doSend({
    			handlerName: 'checkSoftCert',
    			data: {}
    		}, responseCallback);
    }

	function signDataBySoftCert(data, responseCallback) {
    		_doSend({
    			handlerName: 'signDataBySoftCert',
    			data: data
    		}, responseCallback);
    }

	function getSoftCertInfo(responseCallback) {
    		_doSend({
    			handlerName: 'getSoftCertInfo',
    			data: {}
    		}, responseCallback);
    }

	var SoftbankJsSdk = window.SoftbankJsSdk = {
		init: init,
		send: send,
		registerHandler: registerHandler,
		callHandler: callHandler,
		getSdkVersion: getSdkVersion,
		showKeyboard: showKeyboard,
		closeKeyboard: closeKeyboard,
		closeWindow: closeWindow,
		scanQRCode: scanQRCode,
		scanIdCard: scanIdCard,
//		scanBankCard: scanBankCard,
		showSpecifyActivity: showSpecifyActivity,
		identifyVerify: identifyVerify,
		initStyle: initStyle,
		goBackOrForward: goBackOrForward,
		setSofeKeyboardInputListener: setSoftKeyboardInputListener,
		getBankBaseInfo: getBankBaseInfo,
		requestToken: requestToken,
		requestTokenOrRedirect: requestTokenOrRedirect,
		checkLogonState: checkLogonState,
		requestLogonAndRedirect: requestLogonAndRedirect,
		post: post,
		logoutPer:logoutPer,
		share: share,
		doLivenessDetect: doLivenessDetect,
		push: push,
		gainPayPassword: gainPayPassword,
		checkSoftCert: checkSoftCert,
		signDataBySoftCert: signDataBySoftCert,
		getSoftCertInfo: getSoftCertInfo,
		_fetchQueue: _fetchQueue,
		_handleMessageFromNative: _handleMessageFromNative
	};

	var doc = document;
	_createQueueReadyIframe(doc);
	var readyEvent = doc.createEvent('Events');
	readyEvent.initEvent('SoftbankJsSdkReady');
	readyEvent.bridge = SoftbankJsSdk;
	doc.dispatchEvent(readyEvent);
})();